/*
 * Copyright (c) 2003 wlan.kewl.org Project
 * All rights reserved.
 *
 * Licensed under the terms of the BSD license, see file LICENSE
 * for details.
 *
 * $Id: bin2h.c,v 1.6 2003/11/18 11:48:01 darron Exp $
 */

/*
 * create firmware image
 *
 * 1) validate size
 * 2) validate checksum
 * 3) convert to big endian (arm processor) order
 * 4) generate 32bit word include file
 */

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/uio.h>
#include <unistd.h>
#include <fcntl.h>

int
main(int argc, char **argv)
{
    struct stat st;
    u_int8_t *mem, *data;
    u_int32_t value;
    u_int32_t chksum1, chksum2, size;
    int fd, i, byte;
    
    if( argc!=3 )
    {
        printf("try: %s output firmware\n", argv[0]);
        exit(0);
    }

    if( stat(argv[2], &st) != 0 )
        err(1,"stat failed");

    if( (mem=(u_int8_t *)calloc(st.st_size, sizeof(u_int8_t))) == NULL)
        err(2,"calloc failed");

    if( (fd=open(argv[2],O_RDONLY)) <0 )
        err(4,"open failed");

    if( (i=read(fd,(void *)mem,st.st_size))!=st.st_size)
    {
        printf("read only %d bytes", i);
        err(5,"read failed");
    }
    close(fd);

    chksum1=((u_int32_t *)mem)[0];
    size=   ((u_int32_t *)mem)[1];

    if( size!=(st.st_size-8) )
        err(6, "invalid size");

    chksum2=0;
    for(i=4; i<st.st_size; i++)
        chksum2+=mem[i];

    if( chksum1!=chksum2 )
        err(7,"checksum failed");

    printf("/* firmware: %s */\n", argv[2]);
    printf("/* filesize: %d */\n", st.st_size);
    printf("/* checksum: %08x */\n", chksum1);
    printf("/* datasize: %d */\n", size);
    printf("const u_int32_t %s[]= {\n", argv[1]);

    data=&mem[8];
    byte=0;
    for(i=0; i<size; i+=4)
    {
        value= data[i] << 24 | data[i+1] << 16 | data[i+2] << 8 | data[i+3];
        printf("0x%08x,", value);
        if( (++byte % 4)==0 )
            printf("\n");
    }

    printf("\n};\n");

    printf("/* checksum: %08x */\n", chksum2);

    free(mem);

    exit(0);
}
