/*
 * Copyright (c) 2003 wlan.kewl.org Project
 * All rights reserved.
 *
 * Licensed under the terms of the BSD license, see file LICENSE
 * for details.
 *
 * $Id: if_acxeeprom.c,v 1.16 2004/01/28 18:55:05 darron Exp $
 */

/* system */
#include "if_acxsys.h"

/* declarations */
#include "if_acx.h"
#include "if_acxreg.h"
#include "if_acxeeprom.h"

/* functions */
#include "if_acxreg_proto.h"
#include "if_acxeeprom_proto.h"
 
/*
 * regulatory domain info
 */
struct acx_eeprom_domain acx_eeprom_domains[]=
{
    {0x10, "FCC USA (1-11)",     1,11},
    {0x20, "DOC Canada (1-11)",  1,11},
    {0x30, "ETSI Europe (1-13)", 1,13},
    {0x32, "Spain (10-11)",     10,11},
    {0x32, "France (10-13)",    10,13},
    {0x40, "MKK Japan (14)",    14,14},
    {0x41, "MKK1 Japan (1-14)",  1,14},
    {-1,   "(null)", 0, 0}
};

/*
 * determine domain
 */
void
acx_eeprom_init_domain(struct acx_softc *sc)
{
    int i;

    for(i=0; acx_eeprom_domains[i].id!=-1; i++)
        if( acx_eeprom_domains[i].id == sc->eeprom_domain )
            break;

    if( acx_eeprom_domains[i].id==-1) /* not found, set to default */
        sc->eeprom_domain=acx_eeprom_domains[i=0].id;

    sc->eeprom_domainstr= acx_eeprom_domains[i].idstr;
    sc->eeprom_domainlow= acx_eeprom_domains[i].low;
    sc->eeprom_domainhigh=acx_eeprom_domains[i].high;
}

/*
 * read word in eeprom
 */
inline u_int16_t
acx_eeprom_read(struct acx_softc *sc, u_int16_t address)
{
    int temp;

    /*
     * specify address
     */
    WRITEW(&sc->mem1, ACX100_EEPROM_3, 0x0);
    WRITEW(&sc->mem1, ACX100_EEPROM_4, 0x0);
    WRITEW(&sc->mem1, ACX100_EEPROM_ADDR, address);
    WRITEW(&sc->mem1, ACX100_EEPROM_2, 0x0);
    WRITEW(&sc->mem1, ACX100_EEPROM_0, 0x2);
    WRITEW(&sc->mem1, ACX100_EEPROM_1, 0x0);
        
    /*
     * wait for chip to be ready
     */
    temp=0;
    while( READW(&sc->mem1, ACX100_EEPROM_0) )
    {
        if(temp++ == 100)
        {
            device_printf(sc->dev, "acx_eeprom_read: problem: timeout reading word\n");
            return 0; /* XXX return with bogus data */
        }
        DELAY(10000); /* 10ms */
    }
    
    /*
     * get value
     */
    return READW(&sc->mem1, ACX100_EEPROM_DATA);
}

/*
 * init eeprom
 */
void
acx_eeprom_init(struct acx_softc *sc)
{
    /*
    u_int16_t i;
    u_int8_t tmp[0x3b];
    */

    /*
     * read lots of values for some reason
     *
     * notes:
     *  dump this out and see what it is.
     *  it is: Wireless Network CardBus PC Card.Global.........#...$...q.^.
     */
    /*
    for(i=0x8c; i<0xb9; i++)
        tmp[i-0x8c]= (u_int8_t) acx_eeprom_read(sc, i);
     */

    /*
     * read eeprom version
     */
    sc->eeprom_version= acx_eeprom_read(sc, 0x0005);

    /*
     * read eeprom domain
     */
    if( sc->eeprom_version < 5 )
        sc->eeprom_domain= acx_eeprom_read(sc, 0x016f);
    else
        sc->eeprom_domain= acx_eeprom_read(sc, 0x0171);

    acx_eeprom_init_domain(sc);
}
