/*
 * Copyright (c) 2003 wlan.kewl.org Project
 * All rights reserved.
 *
 * Licensed under the terms of the BSD license, see file LICENSE
 * for details.
 *
 * $Id: if_acxioctl.c,v 1.3 2004/02/26 09:31:28 darron Exp $
 */

/* local ioctl functions to be called from the private tool, plus
 * management of ioctl real time stats */

/*
 * XXX use macros for some stats gathering
 */

/* system */
#include "if_acxsys.h"

/* declarations */
#include "if_acx.h"
#include "if_acxreg.h"
#include "if_acxcmd.h"
#include "if_acxioctl.h"
#include "if_acxpacket.h"

/* functions */
#include "if_acxreg_proto.h"
#include "if_acxioctl_proto.h"
#include "if_acxcmd_proto.h"
#include "if_acxdev_proto.h"

/*
 * iotcl for private acxtool
 *
 * notes:
 *  content copied out is mostly volatile and should not be used for serious application
 *
 *  XXX
 *  should implement an error return.
 *  should implement updating the WEP cache for 232/256 bit keys.
 */
int
acx_ioctl(struct ifnet *ifp, u_long cmd, caddr_t data)
{
    struct acx_softc *sc= ifp->if_softc;
    //struct ieee80211com *ic= &sc->ic;
    //struct ifnet *ifp= &ic->ic_if;

    u_int16_t type;
    int16_t val;
    int16_t len;
    struct ieee80211req *ieee80211req= (struct ieee80211req *) data;

    union
    {
        void *vp;
        u_int8_t *bp;
        u_int16_t *wp;
        u_int32_t *lp;
    } datap;
    
    int error=0;

    /*
     * net80211 ioctl parameter block
     */
    type=     ieee80211req->i_type;
    val=      ieee80211req->i_val;
    len=      ieee80211req->i_len; 
    datap.vp= ieee80211req->i_data;

    switch( type )
    {
        /*
         * SET commands
         */

        /* this has limited use as the RX config is set at association time */
        case ACX_IOCTL_SET_RX_CONFIG:
            acx_cmd_set_rx_config(sc, *datap.lp);
            break;
        
        /* this can be set any time after the driver loads */
        case ACX_IOCTL_SET_MAX_XMIT_MSDU_LIFETIME:
            acx_cmd_set_max_xmit_msdu_lifetime(sc, *datap.lp);
            break;

        /* this can be set any time after the driver loads */
        case ACX_IOCTL_SET_ENERGY_DETECT_THRESHOLD:
            acx_cmd_set_ed_threshold(sc, *datap.bp);
            break;

        /* this can be set any time after the driver loads */
        case ACX_IOCTL_SET_SHORT_RETRY_LIMIT:
            acx_cmd_set_short_retry_limit(sc, *datap.bp);
            break;

        /* this can be set any time after the driver loads */
        case ACX_IOCTL_SET_LONG_RETRY_LIMIT:
            acx_cmd_set_long_retry_limit(sc, *datap.bp);
            break;

        /* this can be set any time after the driver loads */
        case ACX_IOCTL_SET_CURRENT_CCA_MODE:
            acx_cmd_set_current_cca_mode(sc, *datap.bp);
            break;

        /* this can be set any time after the driver loads */
        case ACX_IOCTL_SET_CURRENT_ANTENNA:
            acx_cmd_set_current_antenna(sc, *datap.bp);
            break;
        
        /* this can be set any time after the driver loads */
        case ACX_IOCTL_SET_POWER_LEVEL:
            acx_dev_set_power_level(sc, *datap.bp);
            break;

        /*
         * GET commands
         */
        case ACX_IOCTL_GET_RX_CONFIG:
            acx_cmd_get_rx_config(sc, datap.lp);
            break;
        
        case ACX_IOCTL_GET_MAX_XMIT_MSDU_LIFETIME:
            acx_cmd_get_max_xmit_msdu_lifetime(sc, datap.lp);
            break;

        case ACX_IOCTL_GET_ENERGY_DETECT_THRESHOLD:
            acx_cmd_get_ed_threshold(sc, datap.bp);
            break;
        
        case ACX_IOCTL_GET_SHORT_RETRY_LIMIT:
            acx_cmd_get_short_retry_limit(sc, datap.bp);
            break;

        case ACX_IOCTL_GET_LONG_RETRY_LIMIT:
            acx_cmd_get_long_retry_limit(sc, datap.bp);
            break;

        case ACX_IOCTL_GET_CURRENT_CCA_MODE:
            acx_cmd_get_current_cca_mode(sc, datap.bp);
            break;

        case ACX_IOCTL_GET_CURRENT_ANTENNA:
            acx_cmd_get_current_antenna(sc, datap.bp);
            break;

        /* real time statistics */
        case ACX_IOCTL_GET_STATS:
            copyout(&sc->stats, datap.vp, sizeof(struct acx_ioctl_stats));
            break;

        default:
            error= 0;
            break;
    }

    return error;
}

/*
 * update RX statistics
 */
void
acx_ioctl_update_rx_stats(struct acx_softc *sc, struct acx_packet_tiheader *tiheader)
{
    sc->stats.rx_count++;

    sc->stats.rx_level+= (u_int32_t) tiheader->level;
    sc->stats.rx_levelcurrent= (sc->stats.rx_levelcurrent + tiheader->level) >> 1;

    if( tiheader->level > sc->stats.rx_levelmax )
        sc->stats.rx_levelmax= tiheader->level;

    sc->stats.rx_snr+= (u_int32_t) tiheader->snr;
    sc->stats.rx_snrcurrent= (sc->stats.rx_snrcurrent + tiheader->snr) >> 1;

    if( tiheader->snr > sc->stats.rx_snrmax )
        sc->stats.rx_snrmax= tiheader->snr;

    if( sc->stats.rx_count == ACX_IOCTL_STATOVERFLOW )
    {
        sc->stats.rx_count=1;

        sc->stats.rx_level= sc->stats.rx_level >> ACX_IOCTL_STATOVERFLOWSHIFT;
        sc->stats.rx_snr= sc->stats.rx_snr >> ACX_IOCTL_STATOVERFLOWSHIFT;
    }

    /* nb. rx packet counter may not always be the same as rx_count */
    sc->stats.rx_packetcount++;
}
