/*
 * Copyright (c) 2003 wlan.kewl.org Project
 * All rights reserved.
 *
 * Licensed under the terms of the BSD license, see file LICENSE
 * for details.
 *
 * $Id: if_acxutil.c,v 1.5 2004/02/26 10:50:32 darron Exp $
 */

/* utilities */

/* system */
#include "if_acxsys.h"

/* declarations */
#include "if_acx.h"

/* functions */
#include "if_acxutil_proto.h"

/*
 * malloc a string to contain an ascii representation of a rate,
 * caller must FREE(X, M_TEMP)
 */
char *
acx_util_rate(int rate)
{
    char *ratestring=NULL;

    MALLOC(ratestring, char *, 32, M_TEMP, M_NOWAIT | M_ZERO);
    if(ratestring==NULL)
        return NULL;

    rate&=IEEE80211_RATE_VAL;

    if(rate&1) /* fractional rate (eg. 5.5) */
        sprintf(ratestring, "%d.5Mbps", rate>>1);
    else
        sprintf(ratestring, "%dMbps",   rate>>1);

    return ratestring;
}

/*
 * malloc a string to contain an ascii representation of all rates,
 * caller must FREE(X, M_TEMP)
 */
char *
acx_util_rates(struct ieee80211_rateset *rates)
{
    char *rates_string=NULL, *ratestring=NULL;
    int i, j=0;

    MALLOC(rates_string, char *, 128, M_TEMP, M_NOWAIT | M_ZERO);
    if(rates_string==NULL)
        return NULL;

    for(i=0;j<100 && i<rates->rs_nrates;i++)
    {
        ratestring= acx_util_rate(rates->rs_rates[i]);
        sprintf(&rates_string[j], " %s", ratestring);
        j+=1+strlen(ratestring);
        FREE(ratestring, M_TEMP);
    }

    return rates_string;
}
