/*
 * Copyright (c) 2003 wlan.kewl.org Project
 * All rights reserved.
 *
 * Licensed under the terms of the BSD license, see file LICENSE
 * for details.
 *
 * $Id: if_acxcmd.h,v 1.92 2004/01/28 01:29:30 darron Exp $
 */
#ifndef _IF_ACXCMD_H
#define _IF_ACXCMD_H

/*
 * misc defines
 */
#define ACX_CMD_BLOCK NULL
#define ACX_CMD_BLOCK_TIMEOUT (hz*5)

#define ACX_CMD_ERR_RESETFAILED -1
#define ACX_CMD_ERR_UPLOADFAILED -2

/*
 * interrogate/configure structs
 */

/*
 * memory map
 */
#define ACX100_RID_MEMORY_MAP                   0x08
struct acx_cmd_memory_map_param
{
    u_int32_t CodeStart;
    u_int32_t CodeEnd;
    u_int32_t WEPCacheStart;
    u_int32_t WEPCacheEnd;
    u_int32_t PacketTemplateStart;
    u_int32_t PacketTemplateEnd;
    u_int32_t QueueStart;
    u_int32_t QueueEnd;
    u_int32_t MemoryMap1;
    u_int32_t MemoryMap2;
};

/*
 * wep options
 */
#define ACX100_RID_WEP_OPTIONS                  0x07
struct acx_cmd_wep_options_param
{
    u_int16_t word0;
    u_int8_t mode;
};

/*
 * rx config options
 */
#define ACX100_RID_RXCONFIG                     0x10
struct acx_cmd_rx_config_param
{
    u_int32_t options;
};
                                      /*0x00000001     bit00: UNUSED */
#define ACX_CMD_RXCFG_INCLUDE_ADDIT_HDR 0x00000002  /* bit01: DO NOT USE, include additional header (extended/chksum) */
#define ACX_CMD_RXCFG_INCLUDE_FCS       0x00000004  /* bit02: include checksum */
#define ACX_CMD_RXCFG_RCV_ALL_FRAMES    0x00000008  /* bit03: receive all frames */

#define ACX_CMD_RXCFG_UNPROMISCUOUS     0x00000010  /* bit04: unpromiscuous mode */
#define ACX_CMD_RXCFG_FILTER_BSSID      0x00000020  /* bit05: bssid filter */
                                      /*0x00000040     bit06: unknown */
                                      /*0x00000080     bit07: unknown */

                                      /*0x00000100     bit08: unknown */
#define ACX_CMD_RXCFG_DISABLE_BCAST     0x00000200  /* bit09: discard broadcast */
#define ACX_CMD_RXCFG_ONLY_OWN_BEACONS  0x00000400  /* bit10: receive only beacon frames from us */
                                      /*0x00000800     bit11: ALWAYS SET, unknown */

                                      /*0x00001000     bit12: UNUSED */
#define ACX_CMD_RXCFG_PLUS_ADDIT_HDR    0x00002000  /* bit13: ALWAYS USE, include additional header (standard/stats) */
                                      /*0x00004000     bit14: UNUSED */
                                      /*0x00008000     bit15: UNUSED */

#define ACX_CMD_RXCFG_RCV_OTHER         0x00010000  /* bit16: receive other */
#define ACX_CMD_RXCFG_RCV_ACK_FRAMES    0x00020000  /* bit17: receive ack frames */
#define ACX_CMD_RXCFG_RCV_PROBE_RESP    0x00040000  /* bit18: receive probe responses */
#define ACX_CMD_RXCFG_RCV_PROBE_REQ     0x00080000  /* bit19: receive probe requests */

#define ACX_CMD_RXCFG_RCV_MGMT_FRAMES   0x00100000  /* bit20: receive management frames */
#define ACX_CMD_RXCFG_RCV_BROKEN_FRAMES 0x00200000  /* bit21: receive broken frames */
#define ACX_CMD_RXCFG_RCV_DATA_FRAMES   0x00400000  /* bit22: receive data frames */
#define ACX_CMD_RXCFG_RCV_CTRL_FRAMES   0x00800000  /* bit23: receive control frames */

#define ACX_CMD_RXCFG_RCV_UNKNOWN       0x01000000  /* bit24: receive unknown frames */
#define ACX_CMD_RXCFG_RCV_BEACON_FRAMES 0x02000000  /* bit25: receive beacon frames */
#define ACX_CMD_RXCFG_RCV_AUTH_FRAMES   0x04000000  /* bit26: receive auth frames */
#define ACX_CMD_RXCFG_RCV_ASSOC_FRAMES  0x08000000  /* bit27: receive association frames */

                                      /*0x10000000     bit28: UNUSED */
                                      /*0x20000000     bit29: UNUSED */
                                      /*0x40000000     bit30: UNUSED */
                                      /*0x80000000     bit31: UNUSED */

/* infrastructure mode (BSS) 0x0e542000 */
#define ACX_CMD_RXCFG_BSS ( \
        ACX_CMD_RXCFG_UNPROMISCUOUS     | \
        ACX_CMD_RXCFG_PLUS_ADDIT_HDR    | \
        ACX_CMD_RXCFG_RCV_PROBE_RESP    | \
        ACX_CMD_RXCFG_RCV_MGMT_FRAMES   | \
        ACX_CMD_RXCFG_RCV_DATA_FRAMES   | \
        ACX_CMD_RXCFG_RCV_BEACON_FRAMES | \
        ACX_CMD_RXCFG_RCV_AUTH_FRAMES   | \
        ACX_CMD_RXCFG_RCV_ASSOC_FRAMES )

#define ACX_CMD_RXCFG_IBSS ( \
        ACX_CMD_RXCFG_UNPROMISCUOUS     | \
        ACX_CMD_RXCFG_PLUS_ADDIT_HDR    | \
        ACX_CMD_RXCFG_RCV_PROBE_REQ    | \
        ACX_CMD_RXCFG_RCV_PROBE_RESP     | \
        ACX_CMD_RXCFG_RCV_MGMT_FRAMES   | \
        ACX_CMD_RXCFG_RCV_DATA_FRAMES   | \
        ACX_CMD_RXCFG_RCV_BEACON_FRAMES | \
        ACX_CMD_RXCFG_RCV_AUTH_FRAMES   | \
        ACX_CMD_RXCFG_RCV_ASSOC_FRAMES )

#define ACX_CMD_RXCFG_NORMAL_LINUX ( \
        ACX_CMD_RXCFG_RCV_ALL_FRAMES    | \
        ACX_CMD_RXCFG_UNPROMISCUOUS     | \
        ACX_CMD_RXCFG_FILTER_BSSID      | \
        ACX_CMD_RXCFG_ONLY_OWN_BEACONS  | \
        ACX_CMD_RXCFG_PLUS_ADDIT_HDR    | \
        ACX_CMD_RXCFG_RCV_OTHER         | \
        ACX_CMD_RXCFG_RCV_PROBE_RESP    | \
        ACX_CMD_RXCFG_RCV_PROBE_REQ     | \
        ACX_CMD_RXCFG_RCV_MGMT_FRAMES   | \
        ACX_CMD_RXCFG_RCV_DATA_FRAMES   | \
        ACX_CMD_RXCFG_RCV_CTRL_FRAMES   | \
        ACX_CMD_RXCFG_RCV_UNKNOWN       | \
        ACX_CMD_RXCFG_RCV_BEACON_FRAMES | \
        ACX_CMD_RXCFG_RCV_AUTH_FRAMES   | \
        ACX_CMD_RXCFG_RCV_ASSOC_FRAMES )

#define ACX_CMD_RXCFG_MONITOR_LINUX ( \
        ACX_CMD_RXCFG_INCLUDE_FCS       | \
        ACX_CMD_RXCFG_RCV_ALL_FRAMES    | \
        ACX_CMD_RXCFG_UNPROMISCUOUS     | \
        ACX_CMD_RXCFG_PLUS_ADDIT_HDR    | \
        ACX_CMD_RXCFG_RCV_OTHER         | \
        ACX_CMD_RXCFG_RCV_ACK_FRAMES    | \
        ACX_CMD_RXCFG_RCV_PROBE_RESP    | \
        ACX_CMD_RXCFG_RCV_PROBE_REQ     | \
        ACX_CMD_RXCFG_RCV_MGMT_FRAMES   | \
        ACX_CMD_RXCFG_RCV_BROKEN_FRAMES | \
        ACX_CMD_RXCFG_RCV_DATA_FRAMES   | \
        ACX_CMD_RXCFG_RCV_CTRL_FRAMES   | \
        ACX_CMD_RXCFG_RCV_UNKNOWN       | \
        ACX_CMD_RXCFG_RCV_BEACON_FRAMES | \
        ACX_CMD_RXCFG_RCV_AUTH_FRAMES   | \
        ACX_CMD_RXCFG_RCV_ASSOC_FRAMES )

/*
 * clear channel assessment
 */
#define ACX100_RID_DOT11_CURRENT_CCA_MODE       0x100e
struct acx_cmd_current_cca_param
{
    u_int8_t mode;
};

/*
 * energy detect threshold
 */
#define ACX100_RID_DOT11_ED_THRESHOLD           0x100f
struct acx_cmd_ed_threshold_param
{
    u_int32_t level; /* should this be int8? */
};

/*
 * regulatory domain
 */
#define ACX100_RID_DOT11_CURRENT_REG_DOMAIN     0x100a
struct acx_cmd_current_reg_param
{
    u_int8_t domain;
};

/*
 * antenna selection
 */
#define ACX100_RID_DOT11_CURRENT_ANTENNA        0x100b
struct acx_cmd_current_antenna_param
{
    u_int8_t selection;
};

/*
 * wep key
 */
#define ACX100_RID_DOT11_WEP_KEY                0x1007
struct acx_cmd_wep_key_param
{                
    u_int8_t    byte0;
    u_int8_t    length;
    u_int8_t    index;                                                                               
    u_int8_t    key[ACX_WEP_KEYSIZE];                                                                                               
};

/*
 * wep key selection
 */
#define ACX100_RID_DOT11_WEP_DEFAULT_KEY_SET    0x1010
struct acx_cmd_wep_key_default_param
{
    u_int8_t    index;
};

/*
 * firmware rx/tx queue configuration
 */
#define ACX100_RID_QUEUE_CONFIG                 0x03
struct acx_cmd_queue_config_param
{
    u_int32_t AreaSize;
    u_int32_t RxQueueStart;
    u_int8_t  vald;         /* always 1 */
    u_int8_t  vale;         /* always 1 */
    u_int16_t valf;
    u_int32_t QueueEnd;
    u_int32_t QueueEnd2;
    u_int32_t TxQueueStart;
    u_int8_t  valj;         /* always 0 */
    u_int8_t  valk;
    u_int16_t vall;
};

/*
 * memory block size
 */
#define ACX100_RID_BLOCK_SIZE                   0x04
struct acx_cmd_block_size_param
{
    u_int16_t size;
};

/*
 * dma regions
 */
#define ACX100_RID_MEMORY_CONFIG_OPTIONS        0x05
struct acx_cmd_memory_config_options_param
{
    u_int32_t DMA_config;           // always 0x00030000
    bus_addr_t PoolStartAddress;    // physical address of rx descriptors
    u_int32_t val0xc;
    u_int32_t val0x10;
    u_int16_t TxBlockNum;
    u_int16_t RxBlockNum;
};

/*
 * firmware version/revision
 */
#define ACX100_RID_FWREV                        0x0d
struct acx_cmd_firmware_version_param
{
    u_int8_t version[0x18];
};

/*
 * ssid
 */
#define ACX100_RID_SSID                         0x08    /* huh? */
struct acx_cmd_ssid_param
{
    u_int8_t ssid[IEEE80211_NWID_LEN]; /* no terminator */
};

/*
 * msdu lifetime
 */
#define ACX100_RID_DOT11_MAX_XMIT_MSDU_LIFETIME 0x1008
struct acx_cmd_max_xmit_msdu_lifetime_param
{
    u_int32_t lifetime;
};

/*
 * short retry limit
 */
#define ACX100_RID_DOT11_SHORT_RETRY_LIMIT      0x1005
struct acx_cmd_short_retry_limit_param
{
    u_int8_t limit;
};

/*
 * long retry limit
 */
#define ACX100_RID_DOT11_LONG_RETRY_LIMIT       0x1006
struct acx_cmd_long_retry_limit_param
{
    u_int8_t limit;
};

#define ACX100_RID_DOT11_STATION_ID             0x1001
struct acx_cmd_station_id_param
{
    u_int8_t id[6];
};

/*
 * unhandled get/set types
 */
#define ACX100_RID_UNKNOWN_00                   0x00    /* mapped to cfgInvalid in FW150 */
#define ACX100_RID_ACX_TIMER                    0x01
#define ACX100_RID_POWER_MGMT                   0x02
#define ACX100_RID_RATE_FALLBACK                        0x06
#define ACX100_RID_UNKNOWN_09                   0x09    /* mapped to cfgInvalid in FW150 */
#define ACX100_RID_ASSOC_ID                     0x0a
#define ACX100_RID_UNKNOWN_0B                   0x0b    /* mapped to cfgInvalid in FW150 */
#define ACX100_RID_UNKNOWN_0C                   0x0c    /* very small implementation in FW150! */
#define ACX100_RID_FCS_ERROR_COUNT              0x0e
#define ACX100_RID_MEDIUM_USAGE                 0x0f
#define ACX100_RID_UNKNOWN_11                   0x11    /* large implementation in FW150! link quality readings or so? */
#define ACX100_RID_UNKNOWN_12                   0x12    /* VERY large implementation in FW150!! maybe monitor mode?  ?? :-) */
#define ACX100_RID_FIRMWARE_STATISTICS          0x13
#define ACX100_RID_DOT11_UNKNOWN_1002           0x1002  /* mapped to cfgInvalid in FW150 */
#define ACX100_RID_DOT11_BEACON_PERIOD          0x1003  /* mapped to cfgInvalid in FW150 */
#define ACX100_RID_DOT11_DTIM_PERIOD            0x1004  /* mapped to cfgInvalid in FW150 */
#define ACX100_RID_DOT11_UNKNOWN_1009           0x1009  /* mapped to some very boring binary table in FW150 */
#define ACX100_RID_DOT11_UNKNOWN_100C           0x100c  /* mapped to cfgInvalid in FW150 */
#define ACX100_RID_DOT11_TX_POWER_LEVEL         0x100d
#define ACX100_RID_DOT11_UNKNOWN_1011           0x1011  /* mapped to cfgInvalid in FW150 */
#define ACX100_RID_DOT11_UNKNOWN_1012           0x1012  /* mapped to cfgInvalid in FW150 */
#define ACX100_RID_DOT11_UNKNOWN_1013           0x1013  /* mapped to cfgInvalid in FW150 */
//#define ACX100_RID_ACX_TIMER_LEN                0x10
//#define ACX100_RID_POWER_MGMT_LEN               0x06
//#define ACX100_RID_QUEUE_CONFIG_LEN             0x1c
//#define ACX100_RID_BLOCK_SIZE_LEN               0x02
//#define ACX100_RID_MEMORY_CONFIG_OPTIONS_LEN    0x14
//#define ACX100_RID_RATE_FALLBACK_LEN            0x01
//#define ACX100_RID_WEP_OPTIONS_LEN              0x03
//#define ACX100_RID_MEMORY_MAP_LEN               0x28
//#define ACX100_RID_SSID_LEN                     0x20
//#define ACX100_RID_ASSOC_ID_LEN                 0x02
//#define ACX100_RID_FWREV_LEN                    0x18
//#define ACX100_RID_FCS_ERROR_COUNT_LEN          0x04
//#define ACX100_RID_MEDIUM_USAGE_LEN             0x08
//#define ACX100_RID_RXCONFIG_LEN                 0x04
//#define ACX100_RID_FIRMWARE_STATISTICS_LEN      0x9c
//#define ACX100_RID_DOT11_STATION_ID_LEN         0x06
//#define ACX100_RID_DOT11_BEACON_PERIOD_LEN      0x02
//#define ACX100_RID_DOT11_DTIM_PERIOD_LEN        0x01
//#define ACX100_RID_DOT11_SHORT_RETRY_LIMIT_LEN  0x01
//#define ACX100_RID_DOT11_LONG_RETRY_LIMIT_LEN   0x01
//#define ACX100_RID_DOT11_WEP_KEY_LEN            0x09
//#define ACX100_RID_DOT11_WEP_DEFAULT_KEY_LEN    0x20
//#define ACX100_RID_DOT11_MAX_XMIT_MSDU_LIFETIME_LEN     0x04
//#define ACX100_RID_DOT11_CURRENT_REG_DOMAIN_LEN 0x01
//#define ACX100_RID_DOT11_CURRENT_ANTENNA_LEN    0x01
//#define ACX100_RID_DOT11_TX_POWER_LEVEL_LEN     0x01
//#define ACX100_RID_DOT11_CURRENT_CCA_MODE_LEN   0x01
//#define ACX100_RID_DOT11_ED_THRESHOLD_LEN       0x04
//#define ACX100_RID_DOT11_WEP_DEFAULT_KEY_SET_LEN        0x01

/*
 * commands
 */

/* unknown (unknown parameters) */
#define ACX100_CMD_RESET                      0x00

/* interrogate resource (get) */
#define ACX100_CMD_INTERROGATE                0x01

/* configure  resource (set) */
#define ACX100_CMD_CONFIGURE                  0x02
struct acx_cmd_resource_param
{
    u_int16_t type;         /* resource type */
    u_int16_t length;           /* param length */
    union
    {
        struct acx_cmd_memory_map_param memory_map;
    	struct acx_cmd_wep_options_param wep_options;
        struct acx_cmd_rx_config_param rx_config;
        struct acx_cmd_current_cca_param current_cca;
        struct acx_cmd_ed_threshold_param ed_threshold;
        struct acx_cmd_current_reg_param current_reg;
        struct acx_cmd_current_antenna_param current_antenna;
        struct acx_cmd_wep_key_param wep_key;
        struct acx_cmd_wep_key_default_param wep_key_default;
        struct acx_cmd_queue_config_param queue_config;
        struct acx_cmd_block_size_param block_size;
        struct acx_cmd_memory_config_options_param memory_config_options;
        struct acx_cmd_ssid_param ssid;
        struct acx_cmd_max_xmit_msdu_lifetime_param lifetime;
        struct acx_cmd_short_retry_limit_param short_limit;
        struct acx_cmd_long_retry_limit_param long_limit;
        struct acx_cmd_firmware_version_param firmware_version;
        struct acx_cmd_station_id_param macaddr;
    } param;
};

/* enable receive */
#define ACX100_CMD_ENABLE_RX                  0x03
struct acx_cmd_enable_rx_param
{
    u_int8_t channel;
};

/* enable transmit */
#define ACX100_CMD_ENABLE_TX                  0x04
struct acx_cmd_enable_tx_param
{
    u_int8_t channel;
};

/* disable receive (no parameters) */
#define ACX100_CMD_DISABLE_RX                 0x05

/* disable transmit (no parameters) */
#define ACX100_CMD_DISABLE_TX                 0x06

/* flush buffers (no parameters) */
#define ACX100_CMD_FLUSH_QUEUE                0x07

/* perform scan */
#define ACX100_CMD_SCAN                       0x08
struct acx_cmd_scan_param
{
    u_int16_t count;            /* number of scans to do */
    u_int16_t start_chan;
    u_int16_t flags;
    u_int8_t max_rate;          /* max probe rate */
    u_int8_t options;
    u_int16_t chan_duration;
    u_int16_t max_probe_delay;
};

/* stop scan (no parameters) */
#define ACX100_CMD_STOP_SCAN                  0x09

/* configure tim */
#define ACX100_CMD_CONFIG_TIM                 0x0a
struct acx_cmd_config_tim_param
{
    u_int16_t size;
    u_int8_t packet[0x100];
};

/* join network */
#define ACX100_CMD_JOIN                       0x0b
struct acx_cmd_join_param
{
    u_int8_t bssid[IEEE80211_ADDR_LEN];
    u_int16_t beacon_interval;
    u_int8_t dtim_interval;
    u_int8_t rates_basic;
    u_int8_t rates_supported;
    u_int8_t rate_tx;
    u_int8_t preamble_type;
    u_int8_t macmode;
    u_int8_t channel;
    u_int8_t essid_len;
    u_int8_t essid[IEEE80211_NWID_LEN];
};
#define ACX_CMD_MACMODE_NONE      0
#define ACX_CMD_MACMODE_IBSS_STA  1
#define ACX_CMD_MACMODE_ESS_STA   2
#define ACX_CMD_MACMODE_ESS_AP    3

/* configure WEP */
#define ACX100_CMD_WEP_MGMT                   0x0c
struct acx_cmd_wep_mgmt_param
{
    u_int8_t addr[IEEE80211_ADDR_LEN];
    u_int8_t vala;
    u_int8_t len;
    u_int8_t key[32];           /* 29*8 == 232bits == WEP256 */
};

/* unknown (unknown parameters) */
#define ACX100_CMD_HALT                       0x0e

/* sleep (no parameters) */
#define ACX100_CMD_SLEEP                      0x0f

/* wakeup (no parameters) */
#define ACX100_CMD_WAKE                       0x10

/* unknown (unknown parameters) */
#define ACX100_CMD_UNKNOWN_11                 0x11

/* initialse memory (no parameters) */
#define ACX100_CMD_INIT_MEMORY                0x12

/* configure beacon */
#define ACX100_CMD_CONFIG_BEACON              0x13
struct acx_cmd_config_beacon_param
{
    u_int16_t size;
    u_int8_t packet[0x100];
};

/* config probe response? */
#define ACX100_CMD_CONFIG_PROBE_RESPONSE      0x14
struct acx_cmd_config_probe_response_param
{
    u_int16_t size;
    u_int8_t packet[0x100];
};

/* unknown (unknown parameters) */
#define ACX100_CMD_CONFIG_NULL_DATA           0x15

/* config probe request? */
#define ACX100_CMD_CONFIG_PROBE_REQUEST       0x16
struct acx_cmd_config_probe_request_param
{
    u_int16_t size;
    u_int8_t packet[0x100];
};

/* unknown (unknown parameters) */
#define ACX100_CMD_TEST                       0x17

/* initialise radio */
#define ACX100_CMD_RADIOINIT                  0x18
struct acx_cmd_radioinit_param
{
    u_int32_t offset;
    u_int32_t length;
};

/* parameter message block */
union acx_cmd_param
{
    struct acx_cmd_enable_rx_param              enable_rx;
    struct acx_cmd_enable_tx_param              enable_tx;
    struct acx_cmd_resource_param               resource;
    struct acx_cmd_scan_param                   scan;
    struct acx_cmd_config_tim_param             config_tim;
    struct acx_cmd_join_param                   join;
    struct acx_cmd_wep_mgmt_param               wep_mgmt;
    struct acx_cmd_config_beacon_param          config_beacon;
    struct acx_cmd_config_probe_response_param  config_probe_response;
    struct acx_cmd_config_probe_request_param   config_probe_request;
    struct acx_cmd_radioinit_param              radioinit;
};

/*
 * command queue entry
 */
struct acx_cmd_entry
{
    STAILQ_ENTRY(acx_cmd_entry) acx_cmd_entries;
    /*
     * set to callback function for a non blocking command
     * set to NULL alias ACX_CMD_BLOCK for a blocking command
     */
    void                    (*callback_func)(void *, void *);
    u_int16_t               command_type;
    u_int16_t               command_status;
    const char              *command_status_string;
    u_int16_t               param_length;
    union acx_cmd_param     param;
};

#endif /* !_IF_ACXCMD_H */
