/*
 * Copyright (c) 2003 wlan.kewl.org Project
 * All rights reserved.
 *
 * Licensed under the terms of the BSD license, see file LICENSE
 * for details.
 *
 * $Id: if_acxcmd_proto.h,v 1.59 2004/02/15 02:30:22 darron Exp $
 */

#ifndef _IF_ACXCMD_PROTO_H
#define _IF_ACXCMD_PROTO_H

void acx_cmd_init(struct acx_softc *sc);
void acx_cmd_end(struct acx_softc *sc);
  
inline void acx_cmd_access(struct acx_softc *sc);
inline void acx_cmd_upload(struct acx_softc *sc, u_int32_t offset);
inline void acx_cmd_info(struct acx_softc *sc);   
inline void acx_cmd_set_param(struct acx_softc *sc, struct acx_cmd_entry *entry);
inline void acx_cmd_get_result(struct acx_softc *sc, struct acx_cmd_entry *cmd);
 
int acx_cmd_upload_firmware(struct acx_softc *sc, struct acx_dev_firmware *fw, u_int32_t offset);
void acx_cmd_get_info(struct acx_softc *sc, u_int16_t *cmd_type, u_int16_t *cmd_status, const char **cmd_status_string);
void acx_cmd_set_type(struct acx_softc *sc, u_int16_t cmd_type);
void acx_cmd_set_status(struct acx_softc *sc, u_int16_t cmd_status);
void acx_cmd_run(struct acx_softc *sc);

void acx_cmd_complete(struct acx_softc *sc, struct acx_cmd_entry *cmd);
void acx_cmd_start(struct acx_softc *sc, struct acx_cmd_entry *cmd);
    
void acx_cmd_exec(struct acx_softc *sc, struct acx_cmd_entry *entry);
void acx_cmd_issue(struct acx_softc *sc, struct acx_cmd_entry *entry);

void acx_cmd_task_irq(struct acx_softc *sc, int count);
void acx_cmd_task_exec(struct acx_softc *sc, int count);

struct acx_cmd_entry *acx_cmd_create(struct acx_softc *sc, u_int16_t command, void *callback_func, u_int16_t param_len);


void acx_cmd_set_timer(struct acx_softc *sc);
void acx_cmd_set_timer_callback(struct acx_softc *sc, struct acx_cmd_entry *cmd);

void acx_cmd_get_memory_map(struct acx_softc *sc, struct acx_cmd_memory_map_param *memory_map);
void acx_cmd_set_memory_map(struct acx_softc *sc, struct acx_cmd_memory_map_param *memory_map);

void acx_cmd_set_current_cca_mode(struct acx_softc *sc, u_int8_t mode);
void acx_cmd_set_ed_threshold(struct acx_softc *sc, u_int8_t threshold);
void acx_cmd_set_current_reg_domain(struct acx_softc *sc, u_int8_t domain);
void acx_cmd_set_current_antenna(struct acx_softc *sc, u_int8_t select);

void acx_cmd_set_wep_options(struct acx_softc *sc, u_int8_t mode);
void acx_cmd_set_rx_config(struct acx_softc *sc, u_int32_t options);

void acx_cmd_get_fwrev(struct acx_softc *sc);
void acx_cmd_get_station_id(struct acx_softc *sc);

void acx_cmd_sleep(struct acx_softc *sc);
void acx_cmd_wake(struct acx_softc *sc);
void acx_cmd_radioinit(struct acx_softc *sc, u_int32_t radio_offset, u_int32_t radio_size);

void acx_cmd_config_tim(struct acx_softc *sc, u_int8_t *packet, int size);
void acx_cmd_config_beacon(struct acx_softc *sc, u_int8_t *packet, int size);
void acx_cmd_config_probe_response(struct acx_softc *sc, u_int8_t *packet, int size);
void acx_cmd_config_probe_request(struct acx_softc *sc, u_int8_t *packet, int size);

void acx_cmd_reset(struct acx_softc *sc);
void acx_cmd_enable_tx_nb(struct acx_softc *sc, u_int8_t channel);
void acx_cmd_enable_rx_nb(struct acx_softc *sc, u_int8_t channel);
void acx_cmd_disable_tx(struct acx_softc *sc);
void acx_cmd_disable_rx(struct acx_softc *sc);

void acx_cmd_scan(struct acx_softc *sc);
void acx_cmd_stop_scan(struct acx_softc *sc);
void acx_cmd_set_wep_key(struct acx_softc *sc, u_int8_t index);

void acx_cmd_init_memory(struct acx_softc *sc);
void acx_cmd_set_block_size(struct acx_softc *sc, u_int16_t block_size);
void acx_cmd_set_queue_config(struct acx_softc *sc, struct acx_cmd_queue_config_param *queue_config);

void acx_cmd_set_memory_config_options(struct acx_softc *sc, struct acx_cmd_memory_config_options_param *memory_options);

void acx_cmd_set_max_xmit_msdu_lifetime(struct acx_softc *sc, u_int32_t lifetime);
void acx_cmd_set_short_retry_limit(struct acx_softc *sc, u_int8_t limit);
void acx_cmd_set_long_retry_limit(struct acx_softc *sc, u_int8_t limit);

void acx_cmd_set_ssid(struct acx_softc *sc, char *nwid, int nwidlen);
void acx_cmd_join_nb(struct acx_softc *sc, u_int8_t macmode);

void acx_cmd_generic_callback(struct acx_softc *sc, struct acx_cmd_entry *cmd);

void acx_cmd_get_current_cca_mode(struct acx_softc *sc, u_int8_t *mode);
void acx_cmd_get_current_antenna(struct acx_softc *sc, u_int8_t *select);
void acx_cmd_get_rx_config(struct acx_softc *sc, u_int32_t *options);
void acx_cmd_get_ed_threshold(struct acx_softc *sc, u_int8_t *threshold);
void acx_cmd_get_max_xmit_msdu_lifetime(struct acx_softc *sc, u_int32_t *lifetime);
void acx_cmd_get_short_retry_limit(struct acx_softc *sc, u_int8_t *limit);
void acx_cmd_get_long_retry_limit(struct acx_softc *sc, u_int8_t *limit);

void acx_cmd_set_wep_tx_key(struct acx_softc *sc, u_int8_t txkey);

#endif /* !_IF_ACXCMD_PROTO_H */
