/*
 * Copyright (c) 2003 wlan.kewl.org Project
 * All rights reserved.
 *
 * Licensed under the terms of the BSD license, see file LICENSE
 * for details.
 *
 * $Id: if_acxioctl.h,v 1.3 2004/02/26 09:31:28 darron Exp $
 */

#ifndef _IF_ACXIOCTL_H
#define _IF_ACXIOCTL_H

/*
 * ioctl
 *
 * notes:
 *  command id 150 has been chosen in the hope that it isn't used
 *  by the driver interface elsewhere. there are no docs about how
 *  to choose an id.
 *
 *                          group num  size
 */
#define ACX_IOCTL_SGIOC _IOWR('i', 150, struct ieee80211req)

#define ACX_IOCTL_SET_RX_CONFIG                 0x0001
#define ACX_IOCTL_SET_MAX_XMIT_MSDU_LIFETIME    0x0002
#define ACX_IOCTL_SET_ENERGY_DETECT_THRESHOLD   0x0003
#define ACX_IOCTL_SET_SHORT_RETRY_LIMIT         0x0004
#define ACX_IOCTL_SET_LONG_RETRY_LIMIT          0x0005
#define ACX_IOCTL_SET_CURRENT_CCA_MODE          0x0006
#define ACX_IOCTL_SET_POWER_LEVEL               0x0007
#define ACX_IOCTL_SET_CURRENT_ANTENNA           0x0008

#define ACX_IOCTL_GET_STATS                     0x1000

#define ACX_IOCTL_GET_RX_CONFIG                 0x1001
#define ACX_IOCTL_GET_MAX_XMIT_MSDU_LIFETIME    0x1002
#define ACX_IOCTL_GET_ENERGY_DETECT_THRESHOLD   0x1003
#define ACX_IOCTL_GET_SHORT_RETRY_LIMIT         0x1004
#define ACX_IOCTL_GET_LONG_RETRY_LIMIT          0x1005
#define ACX_IOCTL_GET_CURRENT_CCA_MODE          0x1006
#define ACX_IOCTL_GET_POWER_LEVEL               0x1007
#define ACX_IOCTL_GET_CURRENT_ANTENNA           0x1008

/*
 * ioctl real time statistics
 */
struct acx_ioctl_stats
{
    /*
     * signal level/noise/packet statistics
     */
    u_int32_t rx_count;             /* number of rx level/snr samples (24bit) */

    u_int32_t rx_level;             /* sum of rx levels (overflows @ 2^32 / 2^8 samples) */
    u_int8_t  rx_levelcurrent;      /* current rx level (average of last 2 samples) */
    u_int8_t  rx_levelmax;          /* maximum level */

    u_int32_t rx_snr;               /* sum of rx snr (overflows @ 2^32 / 2^8 samples) */
    u_int8_t  rx_snrcurrent;        /* current rx snr (average of last 2 samples) */
    u_int8_t  rx_snrmax;            /* maximum level */

    u_int32_t rx_packetcount;       /* total number of received packets */
    u_int32_t rx_errorcount;        /* number of above packets which are erroneous (too short or too long) */

    u_int32_t tx_packetcount;       /* total number of tranmitted packets (32bit) */
    u_int32_t tx_errorcount;        /* number of above packets which are erroneous (transmit failure) */

    u_int32_t rx_bytes;             /* bytes received */
    u_int32_t tx_bytes;             /* bytes sent */
};
#define ACX_IOCTL_STATOVERFLOWSHIFT 24
#define ACX_IOCTL_STATOVERFLOW 16777216 /* if we have this many samples, we need to shift the
                                           current sum >> 24 and reset the counter to one sample */

#define ACX_IOCTL_UPDATE_RX_ERRORS(SC) (SC->stats.rx_errorcount++);
#define ACX_IOCTL_UPDATE_RX_BYTES(SC,LEN) (SC->stats.rx_bytes+=(LEN));

#define ACX_IOCTL_UPDATE_TX_STATS(SC) (SC->stats.tx_packetcount++);

#define ACX_IOCTL_UPDATE_TX_ERRORS(SC) (SC->stats.tx_errorcount++);
#define ACX_IOCTL_UPDATE_TX_BYTES(SC,LEN) (SC->stats.tx_bytes+=(LEN));

#endif /* !_IF_ACXIOCTL_H */
