/*
 * Copyright (c) 2003 wlan.kewl.org Project
 * All rights reserved.
 *
 * Licensed under the terms of the BSD license, see file LICENSE
 * for details.
 *
 * $Id: if_acxpacket.h,v 1.42 2004/02/26 10:32:56 darron Exp $
 */

#ifndef _IF_ACXPACKET_H
#define _IF_ACXPACKET_H

#ifndef IEEE80211_MTU
    #define IEEE80211_MTU 1500
#endif

#define ACX_PACKET_BLOCKSIZE 256

#define ACX_PACKET_RINGSIZE 16

#define ACX_PACKET_CTL_SHORT_PREAMBLE   0x01
#define ACX_PACKET_CTL_FIRST_MPDU       0x02
#define ACX_PACKET_CTL_AUTODMA          0x04
#define ACX_PACKET_CTL_RECLAIM          0x08
#define ACX_PACKET_CTL_USED_FOR_TX      0x40
#define ACX_PACKET_CTL_FREE             0x80

/* notes: MPDU= MAC Protocol Data Unit */
#define ACX_PACKET_CTL_INIT (ACX_PACKET_CTL_FREE        | ACX_PACKET_CTL_RECLAIM | ACX_PACKET_CTL_AUTODMA | ACX_PACKET_CTL_FIRST_MPDU)
#define ACX_PACKET_CTL_DONE (ACX_PACKET_CTL_USED_FOR_TX | ACX_PACKET_CTL_FREE)

/* tx rate defines */
#define ACX_PACKET_THRESHOLD (30)    // number of packets before considering a change
#define ACX_PACKET_MIN_TIME (10)            // min time before attempting rate up
#define ACX_PACKET_MAX_TIME (20*60)         // max time before reattempting rate up

/* convert ieee802.11 rates to acx100 firmware rates
 *
 * ieee802.11 rates: 1Mbps  2, 2Mbps  4, 5.5Mbps 11, 11Mbps  22, 22Mbps  44
 * acx rates       : 1Mbps 10, 2Mbps 20, 5.5Mbps 55, 11Mbps 110, 22Mbps 220
 */
#define ACX_PACKET_IEEE2ACXRATE(X) ((X & IEEE80211_RATE_VAL) * 5)

#define ACX_PACKET_STARTS_FAST 1

/*
 * firmware packet descriptors (internal card usage)
 *
 * notes:
 *  should the opportunity arise to analyse these structures,
 *  then fill in the blanks.
 */
struct acx_packet_firmware_txdesc
{
    u_int32_t pNextDesc;                /* 0x00 */
    u_int32_t HostMemPtr;               /* 0x04 */
    u_int32_t AcxMemPtr;                /* 0x08 */
    u_int32_t tx_time;                  /* 0x0c */
    u_int32_t total_length;             /* 0x10 */
    u_int32_t val0x14;                  /* 0x14 */
    u_int32_t val0x18;                  /* 0x18 */
    struct acx_packet_txdesc *host_desc; /* 0x1c */
    u_int32_t val0x20;                  /* 0x20 */
    u_int8_t Ctl;                       /* 0x24 */
    u_int8_t something2;                /* 0x25 */
    u_int8_t error;                     /* 0x26 */
    u_int8_t AckFailures;               /* 0x27 */
    u_int16_t val0x28;                  /* 0x28 */
    u_int8_t rate;                      /* 0x2a */
    u_int8_t val0x2b;                   /* 0x2b */
    u_int32_t val0x2c;                  /* 0x2c */
};  /* 0x30 */

#define ACX_PACKET_REGL_HOSTMEMPTR    0x04
#define ACX_PACKET_REGB_CTL           0x24
#define ACX_PACKET_REGB_ERR           0x26
#define ACX_PACKET_REGB_RATE          0x2a
#define ACX_PACKET_REGL_LENGTH        0x10
#define ACX_PACKET_REGL_TXTIME        0x0c

struct acx_packet_firmware_rxdesc
{
    u_int32_t pNextDesc;                /* 0x00 */
    u_int32_t HostMemPtr;               /* 0x04 */
    u_int8_t val0x8;                    /* 0x08 */
    u_int8_t val0x9;                    /* 0x09 */
    u_int16_t val0xa;                   /* 0x0a */
    u_int32_t val0xc;                   /* 0x0c */
    u_int32_t val0x10;                  /* 0x10 */
    u_int32_t val0x14;                  /* 0x14 */
    u_int32_t val0x18;                  /* 0x18 */
    u_int32_t val0x1c;                  /* 0x1c */
    u_int32_t val0x20;                  /* 0x20 */
    struct acx_packet_rx *val0x24;      /* 0x24 */
    u_int32_t val0x28;                  /* 0x28 */
    u_int32_t val0x2c;                  /* 0x2c */
    u_int32_t val0x30;                  /* 0x30 */
};  /* 0x34 */

/*
 * payload
 */
struct acx_packet_payload
{
    u_int8_t payload[IEEE80211_MTU];
};

/*
 * TI rx header
 */
struct acx_packet_tiheader
{
    u_int32_t status;   /* TI status */
    u_int16_t stat;     /* TI stats */
    u_int8_t level;     /* TI level */
    u_int8_t snr;       /* TI signal/noise ratio */
    u_int32_t time;     /* TI timestamp */
};

/*
 * rx packet (tiheader/header/payload)
 *
 * notes:
 *  this structure is not in use, but exists as a prototype for a received packet
 */
struct acx_packet_rx
{
    struct acx_packet_tiheader tiheader;    /* rxconfig ACX_RXCFG_PLUS_ADDIT_HDR */
    u_int32_t tiextra;                      /* rxconfig ACX_RXCFG_INCLUDE_ADDIT_HDR */
    struct ieee80211_frame frame;           /* ieee80211 frame header */
    u_int8_t payload[IEEE80211_MTU];        /* payload */
};

#ifdef _KERNEL
/*
 * buf or mbuf with m_ext storage plus dma map
 *
 * notes:
 *  buf is used for static tx buffers and mbuf for dynamic rx buffers
 */
#define ACX_PACKET_TYPE_MBUF 0
#define ACX_PACKET_TYPE_BUF 1
struct acx_packet_bufmap
{
    int type;
    union
    {
        struct mbuf *m;
        u_int8_t *b;
    } buf;
    bus_dmamap_t map;
};
#endif /* _KERNEL */

/*
 * tx packet descriptor
 *
 * notes:
 *  unverified
 */
struct acx_packet_txdesc
{
    u_int8_t *data_phy;                         /* 0x00 */
    u_int16_t val0x4;                           /* 0x04 */
    u_int16_t val0x6;                           /* 0x06 */
    u_int16_t Ctl;                              /* 0x08 */
    u_int16_t length;                           /* 0x0a */
    struct acx_packet_txdesc *desc_phy_next;    /* 0x0c */
    struct acx_packet_txdesc *val0x10;          /* 0x10 */
    u_int32_t val0x14;                          /* 0x14 */
    struct acx_packet_txdesc *desc_phy;         /* 0x18 */
    struct acx_packet_firmware_txdesc *val0x1c; /* 0x1c */
    u_int32_t val0x20;                          /* 0x20 */
    u_int8_t *data;                             /* 0x24 */
    u_int16_t val0x28;                          /* 0x28 */
    u_int8_t rate;                              /* 0x2a */
    u_int8_t val0x2b;                           /* 0x2b */
};  /* 0x2c */

#define ACX_PACKET_DESCSIZE (sizeof(struct acx_packet_txdesc))

/*
 * rx packet descriptor
 */
struct acx_packet_rxdesc
{
    u_int8_t *data_phy;                      /* 0x00 */
    u_int32_t HostMemPtr;                       /* 0x04 */
    u_int16_t Ctl;                              /* 0x08 */
    u_int16_t length;                           /* 0x0a */
    struct acx_packet_rxdesc *desc_phy_next;    /* 0x0c */
    u_int32_t val0x10;                          /* 0x10 */
    u_int32_t val0x14;                          /* 0x14 */
    struct acx_packet_rxdesc *desc_phy;         /* 0x18 */
    u_int32_t val0x1c;                          /* 0x1c */
    u_int32_t val0x20;                          /* 0x20 */
    struct mbuf *data;                          /* 0x24 NOT USED */
    u_int8_t val0x28;                           /* 0x28 */
    u_int8_t val0x29;                           /* 0x29 */
    u_int8_t val0x2a;                           /* 0x2a */
    u_int8_t val0x2b;                           /* 0x2b */
};  /* 0x2c */

#endif /* !_IF_ACXPACKET_H */
