/*
 * Copyright (c) 2003 wlan.kewl.org Project
 * All rights reserved.
 *
 * Licensed under the terms of the BSD license, see file LICENSE
 * for details.
 *
 * $Id: acx.cxx,v 1.34 2004/02/26 09:31:28 darron Exp $
 */

#include "acxtool.h"

acx::acx(string interface) : net80211(interface)
{
}

acx::~acx()
{
}

/*
 * SET/GET
 */
void
acx::sgioc(u_int16_t i_type, void *i_data)
{
    struct ieee80211req ieee80211req;
            
    bzero(&ieee80211req, sizeof(ieee80211req));
    strncpy(ieee80211req.i_name,ifname,IFNAMSIZ);

    ieee80211req.i_data = (void *)    i_data;
    ieee80211req.i_type = (u_int16_t) i_type;
    ieee80211req.i_val  = (int16_t)   0;
    ieee80211req.i_len  = (int16_t)   0;

    if( ioctl(sock, ACX_IOCTL_SGIOC, &ieee80211req) < 0 )
        throw strerror(errno);
}

/*
 * SET
 */
void
acx::set_rx_config(u_int32_t rx_config)
{
    cout << "setting rx config to " << base.hex(rx_config) << endl;
    sgioc(ACX_IOCTL_SET_RX_CONFIG, &rx_config);
}

void
acx::set_max_xmit_msdu_lifetime(u_int32_t max_xmit_msdu_lifetime)
{
    cout << "setting max xmit msdu lifetime to " << base.hex(max_xmit_msdu_lifetime) << endl;
    sgioc(ACX_IOCTL_SET_MAX_XMIT_MSDU_LIFETIME, &max_xmit_msdu_lifetime);
}

void
acx::set_ed_threshold(u_int8_t ed_threshold)
{
    cout << "setting energy detect threshold to " << base.hex(ed_threshold) << endl;
    sgioc(ACX_IOCTL_SET_ENERGY_DETECT_THRESHOLD, &ed_threshold);
}

void
acx::set_short_retry_limit(u_int8_t limit)
{
    cout << "setting short retry limit to " << base.hex(limit) << endl;
    sgioc(ACX_IOCTL_SET_SHORT_RETRY_LIMIT, &limit);
}   

void
acx::set_long_retry_limit(u_int8_t limit)
{
    cout << "setting short retry limit to " << base.hex(limit) << endl;
    sgioc(ACX_IOCTL_SET_LONG_RETRY_LIMIT, &limit);
}   

void
acx::set_cca_mode(u_int8_t mode)
{
    cout << "setting current cca mode to " << base.hex(mode) << endl;
    sgioc(ACX_IOCTL_SET_CURRENT_CCA_MODE, &mode);
}   

void
acx::set_power_level(u_int8_t level)
{
    cout << "setting power level to " << base.hex(level) << endl;
    sgioc(ACX_IOCTL_SET_POWER_LEVEL, &level);
}   

void
acx::set_current_antenna(u_int8_t antenna)
{
    cout << "setting current antenna to " << base.hex(antenna) << endl;
    sgioc(ACX_IOCTL_SET_CURRENT_ANTENNA, &antenna);
}   

/*
 * GET
 */
u_int32_t
acx::get_rx_config()
{
    u_int32_t rx_config;
    sgioc(ACX_IOCTL_GET_RX_CONFIG, &rx_config);
    return rx_config;
}

u_int8_t
acx::get_ed_threshold()
{
    u_int8_t edt;
    sgioc(ACX_IOCTL_GET_ENERGY_DETECT_THRESHOLD, &edt);
    return edt;
}

u_int32_t
acx::get_max_xmit_msdu_lifetime()
{
    u_int32_t lifetime;
    sgioc(ACX_IOCTL_GET_MAX_XMIT_MSDU_LIFETIME, &lifetime);
    return lifetime;
}

u_int8_t
acx::get_short_retry_limit()
{
    u_int8_t limit;
    sgioc(ACX_IOCTL_GET_SHORT_RETRY_LIMIT, &limit);
    return limit;
}

u_int8_t
acx::get_long_retry_limit()
{
    u_int8_t limit;
    sgioc(ACX_IOCTL_GET_LONG_RETRY_LIMIT, &limit);
    return limit;
}

u_int8_t
acx::get_current_cca_mode()
{
    u_int8_t mode;
    sgioc(ACX_IOCTL_GET_CURRENT_CCA_MODE, &mode);
    return mode;
}

u_int8_t
acx::get_current_antenna()
{
    u_int8_t antenna;
    sgioc(ACX_IOCTL_GET_CURRENT_ANTENNA, &antenna);
    return antenna;
}

/*
 * Calculate %
 */
double
acx::percentof(double sample, double total)
{
    double f=0; /* % */

    if( total > 0 )
        f= floor( 0.5 + ( 100 * ( sample / total ) ));

    return f;
}

/*
 * SHOW INFO
 */
void
acx::show_info()
{
    /* get stats */
    struct acx_ioctl_stats stats;
    sgioc(ACX_IOCTL_GET_STATS, &stats);
            
    /* show stats */
    cout << "RX Statistics"   << endl;
    cout << "Total Packets   : " << base.ten(stats.rx_packetcount)             << " Errors:   " << base.ten(stats.rx_errorcount) << " (" << percentof(stats.rx_errorcount, stats.rx_packetcount) << "%)" << endl;
    cout << "Total Bytes     : " << base.ten(stats.rx_bytes) <<  endl;
    cout << endl;

    if( stats.rx_count )
    {

    cout << "Signal Level    : " << base.ten((u_int32_t)stats.rx_levelcurrent) << " Average:  " << base.ten(stats.rx_level / stats.rx_count) << " Max: " << base.ten(stats.rx_levelmax) << endl;
    cout << "Signal/Noise    : " << base.ten((u_int32_t)stats.rx_snrcurrent)   << " Average:  " << base.ten(stats.rx_snr   / stats.rx_count) << " Max: " << base.ten(stats.rx_snrmax)   << endl;
    cout << endl;

    cout << "Signal Strength : " << percentof(stats.rx_levelcurrent,256.0)     << "%" << endl;
    cout << "Signal Quality  : " << 100.0-percentof(stats.rx_snrcurrent,256.0) << "%" << endl;
    cout << endl;
    
    }

    cout << "TX Statistics"   << endl;
    cout << "Total Packets   : " << base.ten(stats.tx_packetcount)             << " Errors:   " << base.ten(stats.tx_errorcount) << " (" << percentof(stats.tx_errorcount, stats.tx_packetcount) << "%)" << endl;
    cout << "Total Bytes     : " << base.ten(stats.tx_bytes) <<  endl;
    cout << endl;

    /* show vars */
    u_int32_t rxc;

    cout << "RX Config              : " << base.hex( rxc=get_rx_config()            ) << endl;
    cout << "Energy Detect Threshold: " << base.hex( get_ed_threshold()             ) << endl;
    cout << "Max Xmit MSDU Lifetime : " << base.hex( get_max_xmit_msdu_lifetime()   ) << endl;
    cout << "Short Retry Limit      : " << base.hex( get_short_retry_limit()        ) << endl;
    cout << "Long Retry Limit       : " << base.hex( get_long_retry_limit()         ) << endl;
    cout << "Current CCA Mode       : " << base.hex( get_current_cca_mode()         ) << endl;
    cout << "Current Antenna        : " << base.hex( get_current_antenna()          ) << endl;
    cout << endl;

    const char *rxc_bits[]={
        NULL,
        "       2 ACX_CMD_RXCFG_INCLUDE_ADDIT_HDR",
        "       4 ACX_CMD_RXCFG_INCLUDE_FCS",
        "       8 ACX_CMD_RXCFG_RCV_ALL_FRAMES",
        "      10 ACX_CMD_RXCFG_UNPROMISCUOUS",
        "      20 ACX_CMD_RXCFG_FILTER_BSSID",
        "      40 UNKNOWN",
        "      80 UNKNOWN",
        "     100 UNKNOWN",
        "     200 ACX_CMD_RXCFG_DISABLE_BCAST",
        "     400 ACX_CMD_RXCFG_ONLY_OWN_BEACONS",
        "     800 UNKNOWN",
        NULL,
        "    2000 ACX_CMD_RXCFG_PLUS_ADDIT_HDR",
        NULL,
        NULL,
        "   10000 ACX_CMD_RXCFG_RCV_OTHER",
        "   20000 ACX_CMD_RXCFG_RCV_ACK_FRAMES",
        "   40000 ACX_CMD_RXCFG_RCV_PROBE_RESP",
        "   80000 ACX_CMD_RXCFG_RCV_PROBE_REQ",
        "  100000 ACX_CMD_RXCFG_RCV_MGMT_FRAMES",
        "  200000 ACX_CMD_RXCFG_RCV_BROKEN_FRAMES",
        "  400000 ACX_CMD_RXCFG_RCV_DATA_FRAMES",
        "  800000 ACX_CMD_RXCFG_RCV_CTRL_FRAMES",
        " 1000000 ACX_CMD_RXCFG_RCV_UNKNOWN",
        " 2000000 ACX_CMD_RXCFG_RCV_BEACON_FRAMES",
        " 4000000 ACX_CMD_RXCFG_RCV_AUTH_FRAMES",
        " 8000000 ACX_CMD_RXCFG_RCV_ASSOC_FRAMES",
        NULL,
        NULL,
        NULL,
        NULL};

    cout << "RX Config Summary :" << endl;

    for(int i=0; i<32; i++)
    {
        if( rxc_bits[i]!=NULL )
        {
            if( rxc & 1 )
                cout << " ";
            else
                cout << "!";
            cout << rxc_bits[i] << endl;
        }
        rxc= rxc >> 1;
    }
    cout << endl;
}
