/*
 * Copyright (c) 2003 wlan.kewl.org Project
 * All rights reserved.
 *
 * Licensed under the terms of the BSD license, see file LICENSE
 * for details.
 *
 * $Id: acxtool.cxx,v 1.27 2004/02/26 09:31:28 darron Exp $
 */

#include "acxtool.h"

int usage(char *reason)
{
    cout << "Error: " << reason << endl;

    cout << "Usage: " << "acxtool acx?" << endl;
    cout << "   -r RX_CONFIG" << endl;
    cout << "   -m MAX_XMIT_MSDU_LIFETIME" << endl;
    cout << "   -e ENERGY_DETECT_THRESHOLD" << endl;
    cout << "   -s SHORT_RETRY_LIMIT" << endl;
    cout << "   -l LONG_RETRY_LIMIT" << endl;
    cout << "   -c CCA_MODE" << endl;
    cout << "   -a CURRENT_ANTENNA" << endl;
    cout << "   -p POWER_LEVEL" << endl;

    return EX_USAGE;
}

int main(int argc, char **argv)
{
    if( argc<2 || argv[1][0]=='-' )
        exit(usage("Missing device name"));

    if( strlen(argv[1])<4 || strlen(argv[1])>4 || !(argv[1][0]=='a' && argv[1][1]=='c' && argv[1][2]=='x' && argv[1][3]>='0' && argv[1][3]<='9') )
        exit(usage("Invalid device name"));

    cout << argv[1] << ":" << endl << endl;

    class acx *acxdev= new acx( argv[1] );

    try
    {
        char c;
        opterr= 0;
        optind++;
        while( (c= getopt(argc, argv, "r:m:e:s:l:c:p:a:i:")) != -1)
        {
            switch(c)
            {
                case 'r':   acxdev->set_rx_config( (u_int32_t) strtoul(optarg, NULL, 0) );
                            break;

                case 'm':   acxdev->set_max_xmit_msdu_lifetime( (u_int32_t) strtoul(optarg, NULL, 0) );
                            break;

                case 'e':   acxdev->set_ed_threshold( (u_int8_t) strtoul(optarg, NULL, 0) );
                            break;

                case 's':   acxdev->set_short_retry_limit( (u_int8_t) strtoul(optarg, NULL, 0) );
                            break;
                
                case 'l':   acxdev->set_long_retry_limit( (u_int8_t) strtoul(optarg, NULL, 0) );
                            break;

                case 'c':   acxdev->set_cca_mode( (u_int8_t) strtoul(optarg, NULL, 0) );
                            break;

                case 'p':   acxdev->set_power_level( (u_int8_t) strtoul(optarg, NULL, 0) );
                            break;

                case 'a':   acxdev->set_current_antenna( (u_int8_t) strtoul(optarg, NULL, 0) );
                            break;

                case '?':
                default:    exit( usage("Invalid option") );
                            break;
            }
        }
        argc-=optind;
        argv+=optind;
    
        if( optind == 2 )   /* show device info when no options given */
            acxdev->show_info();
    }
    catch (const char *exception)
    {
        cout << "Error: " << exception << endl;
    }
    catch (...)
    {
        cout << "Error: Unhandled exception" << endl;
    }

    delete acxdev;

    return 0;
}
